@@ -34,10 +34,10 @@
 #include "ldemul.h"
 #include "libiberty.h"
 #include "filenames.h"
-#ifdef ENABLE_PLUGINS
+#if BFD_SUPPORTS_PLUGINS
 #include "plugin-api.h"
 #include "plugin.h"
-#endif /* ENABLE_PLUGINS */
+#endif /* BFD_SUPPORTS_PLUGINS */
 
 bfd_boolean ldfile_assumed_script = FALSE;
 const char *ldfile_output_machine_name = "";
@@ -270,13 +270,15 @@ again:
       return FALSE;
     }
 
+  track_dependency_files (attempt);
+
   /* Linker needs to decompress sections.  */
   entry->the_bfd->flags |= BFD_DECOMPRESS;
 
   /* This is a linker input BFD.  */
   entry->the_bfd->is_linker_input = 1;
 
-#ifdef ENABLE_PLUGINS
+#if BFD_SUPPORTS_PLUGINS
   if (entry->flags.lto_output)
     entry->the_bfd->lto_output = 1;
 #endif
@@ -368,8 +370,8 @@ again:
 				skip = 1;
 			    }
 			  free (arg1);
-			  if (arg2) free (arg2);
-			  if (arg3) free (arg3);
+			  free (arg2);
+			  free (arg3);
 			  break;
 			case NAME:
 			case LNAME:
@@ -378,8 +380,7 @@ again:
 			  free (yylval.name);
 			  break;
 			case INT:
-			  if (yylval.bigint.str)
-			    free (yylval.bigint.str);
+			  free (yylval.bigint.str);
 			  break;
 			}
 		      token = yylex ();
@@ -430,8 +431,8 @@ again:
 	    }
 	}
     }
-success:
-#ifdef ENABLE_PLUGINS
+ success:
+#if BFD_SUPPORTS_PLUGINS
   /* If plugins are active, they get first chance to claim
      any successfully-opened input file.  We skip archives
      here; the plugin wants us to offer it the individual
@@ -445,7 +446,7 @@ success:
       && !no_more_claiming
       && bfd_check_format (entry->the_bfd, bfd_object))
     plugin_maybe_claim (entry);
-#endif /* ENABLE_PLUGINS */
+#endif /* BFD_SUPPORTS_PLUGINS */
 
   /* It opened OK, the format checked out, and the plugins have had
      their chance to claim it, so this is success.  */
@@ -544,7 +545,24 @@ ldfile_open_file (lang_input_statement_type *entry)
       search_arch_type *arch;
       bfd_boolean found = FALSE;
 
-      /* Try to open <filename><suffix> or lib<filename><suffix>.a */
+      /* If extra_search_path is set, entry->filename is a relative path.
+	 Search the directory of the current linker script before searching
+	 other paths. */
+      if (entry->extra_search_path)
+	{
+	  char *path = concat (entry->extra_search_path, slash, entry->filename,
+			       (const char *)0);
+	  if (ldfile_try_open_bfd (path, entry))
+	    {
+	      entry->filename = path;
+	      entry->flags.search_dirs = FALSE;
+	      return;
+	    }
+
+	  free (path);
+	}
+
+      /* Try to open <filename><suffix> or lib<filename><suffix>.a.  */
       for (arch = search_arch_head; arch != NULL; arch = arch->next)
 	{
 	  found = ldfile_open_file_search (arch->name, entry, "lib", ".a");
@@ -573,6 +591,22 @@ ldfile_open_file (lang_input_statement_type *entry)
 		   entry->local_sym_name, ld_sysroot);
 	  else
 	    einfo (_("%P: cannot find %s\n"), entry->local_sym_name);
+
+	  /* PR 25747: Be kind to users who forgot to add the
+	     "lib" prefix to their library when it was created.  */
+	  for (arch = search_arch_head; arch != NULL; arch = arch->next)
+	    {
+	      if (ldfile_open_file_search (arch->name, entry, "", ".a"))
+		{
+		  const char * base = lbasename (entry->filename);
+
+		  einfo (_("%P: note to link with %s use -l:%s or rename it to lib%s\n"),
+			 entry->filename, base, base);
+		  bfd_close (entry->the_bfd);
+		  entry->the_bfd = NULL;
+		  break;
+		}
+	    }
 	  entry->flags.missing_file = TRUE;
 	  input_flags.missing_file = TRUE;
 	}
@@ -771,6 +805,8 @@ ldfile_open_command_file_1 (const char *name, enum script_open_style open_how)
       return;
     }
 
+  track_dependency_files (name);
+
   lex_push_file (ldlex_input_stack, name, sysrooted);
 
   lineno = 1;
